//***************************************************************************
// A P P L I C A T I O N   N O T E   F O R   T H E   A V R   F A M I L Y
//
// Number               : AVR151
// File Name            : "avr151_Master.c"
// Title                : Setup and Use The SPI
// Date                 : 00.09.20
// Version              : 1.0
// Target MCU           : Any AVR with SPI
//
// DESCRIPTION
// This Application note shows how to enable and use the on-board SPI of the 
// AVR's inSlave- and in Slave-Mode. 
//
// This file contains the code of all the examples. To switch between the examples
// you will have to enable the call of the corresponding routines.
// (see the main program at the bottom) 
//
// Modified 2004-10-25 RAA
//
//**************************************************************************

#include <avr/io.h>
#include <avr/interrupt.h>

#define  Error   0x01
#define  Success 0x02
char     TransmitState = 0x00;
char*    TextString    = "AVR communicating via the SPI"+0x00;


// Interrupt Routine Slave Mode (interrupt controlled)
#pragma vector=SPI_STC_vect
void ISR_SPI (void)
{
        if (SPDR != *TextString)                // Check for transmission errors
        {
                TransmitState = Error;  
                SPCR &= ~(1<<SPIE);             // Disable SPI interrupts
        }
        else 
        {
                TextString++;                   // Set Pointer on next Character
                if (*TextString == 0)           // Check for end of String
                {
                        TransmitState = Success;
                        SPCR &= ~(1<<SPIE);     // Disable SPI interrupts
                }
        }
}


// Intialization RoutineSlave Mode (interrupt controlled)
void Init_Slave_IntContr (void)
{
        volatile char IOReg;
        // Set PB6(MISO) as output 
        DDRB    = (1<<PB6);
        // Enable SPI Interrupt and SPI in Slave Mode with SCK = CK/4
        SPCR  = (1<<SPIE)|(1<<SPE);
        IOReg   = SPSR;                         // Clear SPIF bit in SPSR
        IOReg   = SPDR;
        DDRD	= 0xFF;	                // Set Port D as output
        sei();                                 // Enable global interrupts
}



// Receive Routine Slave Mode (interrupt controlled)
void Slave_Receive_IntContr (void)
{
        while (1)
        {
                switch (TransmitState)
                {       // Output Transmit status on Port D
                        case 0x00:    PORTD = 0x00;
                        break;
                        case Error:   PORTD = 0xF0;
                        break;
                        case Success: PORTD = 0xAA;
                        break;
                }
        }
                
}


void main (void)
{

                // Enable next two lines for Example 2
          Init_Slave_IntContr();
          Slave_Receive_IntContr();
}

